import 'package:stateful_provider/process_error.dart';
import 'package:stateful_provider/process_status.dart';
import 'package:stateful_provider/task_provider.dart';
import 'package:uuid/uuid.dart';

class BaseProvider extends TaskProvider {
  final ProcessStatus _status = ProcessStatus();

  Future<T> enqueue<T>(Future<T> Function() task,
      {bool ignoreIfLoading = false}) async {
    if (ignoreIfLoading && isProcessing) return null;

    final job = Uuid();
    processes.add(job);
    _status.add(job);

    final result = await task().catchError((error) {
      _status.catchError(ProcessError(error));
      return null;
    }).whenComplete(() {
      processes.remove(job);
      _status.complete(job);
    });

    return result;
  }
}
