import 'package:stateful_provider/process_error.dart';
import 'package:uuid/uuid.dart';

import 'task_provider.dart';

class ProcessStatus extends TaskProvider {
  factory ProcessStatus() => _singleton;

  static final ProcessStatus _singleton = ProcessStatus._internal();

  ProcessStatus._internal();

  Set<ProcessError> _exceptions = {};

  bool get hasError => _exceptions.isNotEmpty;

  void add(Uuid uuid) {
    processes.add(uuid);
    _rebuildIfNeeded();
  }

  void complete(Uuid uuid) {
    processes.remove(uuid);
    _rebuildIfNeeded();
  }

  bool _isProcessing = false;

  void _rebuildIfNeeded() {
    final current = isProcessing;
    if (current == _isProcessing) return;
    _isProcessing = current;
    notifyListeners();
  }

  bool _handlingError = false;
  ProcessError currentError() {
    _handlingError = true;
    return _exceptions.firstWhere((element) => true, orElse: () => null);
  }

  void catchError(ProcessError error) {
    _exceptions.add(error);
    if (_handlingError) return;
    notifyListeners();
  }

  void consumeError(ProcessError error) {
    _exceptions.remove(error);
    _handlingError = false;
    notifyListeners();
  }
}
