import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:provider/single_child_widget.dart';
import 'package:stateful_provider/process_error.dart';
import 'package:stateful_provider/process_status.dart';
import 'package:stateful_provider/process_status_widget.dart';
import 'package:uuid/uuid.dart';

final uuid = Uuid();
int count = 0;

void main() {
  List<SingleChildWidget> providers = [
    ChangeNotifierProvider(create: (_) => ProcessStatus())
  ];

  runApp(
    MaterialApp(
      theme: ThemeData.dark(),
      home: MultiProvider(
        providers: providers,
        child: Scaffold(
          appBar: AppBar(
            title: Text("Stateful provider"),
          ),
          body: Consumer<ProcessStatus>(
            builder: (context, provider, __) => Stack(
              children: [
                Action(provider),
                ProcessStatusWidget(
                  error: (error, close) {
                    Scaffold.of(context)
                        .showSnackBar(SnackBar(
                          content: Text("$error"),
                          duration: Duration(seconds: 1),
                        ))
                        .closed
                        .then((value) => close());
                  },
                  process: LinearProgressIndicator(),
                )
              ],
            ),
          ),
        ),
      ),
    ),
  );
}

Widget Action(ProcessStatus status) {
  return Center(
      child: Column(
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          RaisedButton(
            child: Text("Add process"),
            onPressed: () => status.add(uuid),
          ),
          RaisedButton(
            child: Text("Remove process"),
            onPressed: () => status.complete(uuid),
          ),
          RaisedButton(
            child: Text("Add error"),
            onPressed: () => status.catchError(ProcessError(
                Exception(["Error# ${count++}"]))),
          ),
        ],
      ),
  );
}
